﻿// Copyright (c) 2019 by E. Lücke

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;

namespace G_Api_Wrapper
{
    /// <summary>
    /// This class provides additional functions
    /// </summary>
    public class GAPI_Helpers
    {

        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// helper function for converting returned char*
        /// </summary>
        /// <param name="ptr"></param>
        /// <returns></returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        public static string PtrToStringUtf8(IntPtr ptr) // aPtr is nul-terminated
        {
            if (ptr == IntPtr.Zero)
                return "";
            int len = 0;
            while (System.Runtime.InteropServices.Marshal.ReadByte(ptr, len) != 0)
                len++;
            if (len == 0)
                return "";
            byte[] array = new byte[len];
            System.Runtime.InteropServices.Marshal.Copy(ptr, array, 0, len);
            return System.Text.Encoding.UTF8.GetString(array);
        }


        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// creates a subarray as subset of a given array
        /// </summary>
        /// <typeparam name="T">array type -> usually Byte</typeparam>
        /// <param name="data">the array</param>
        /// <param name="index">first element of the array</param>
        /// <param name="length">length of the subarray</param>
        /// <returns></returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        public static T[] SubArray<T>(T[] data, int index, int length)
        {
            T[] result = new T[length];
            Array.Copy(data, index, result, 0, length);
            return result;
        }


        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// Creates a single byte array structure. This is useful, when G-API needs a buffer to write a structure into.
        /// </summary>
        /// <typeparam name="T">structure type</typeparam>
        /// <param name="bytes">byte array from C-Dll</param>
        /// <returns></returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        public static T ByteArrayToStructure<T>(byte[] bytes) where T : struct
        {
            T result;
            GCHandle handle = GCHandle.Alloc(bytes, GCHandleType.Pinned);
            try
            {
                result = (T)Marshal.PtrToStructure(handle.AddrOfPinnedObject(), typeof(T));
            }
            finally
            {
                handle.Free();
            }
            return result;
        }


        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
        /// <summary>
        /// function parses one or more structures from an array of bytes
        /// </summary>
        /// <typeparam name="T">structure type [example: GAPI_CAN.CAN_xyz_t]</typeparam>
        /// <param name="bytes">Byte array with data</param>
        /// <param name="numOfStructures">number of structures in the array</param>
        /// <param name="offset">number of data in a structure</param>
        /// <returns> returns an array of structures with given length</returns>
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


        public static T[] ByteArrayToStructureArray<T>(Byte[] bytes, UInt32 numOfStructures, UInt32 offset) where T : struct
        {
            T[] result = new T[numOfStructures];

            Byte[] subData;
            for (UInt32 i = 0; i < numOfStructures; i++)
            {
                subData = SubArray<Byte>(bytes, Convert.ToInt32(i * offset), Convert.ToInt32(offset));

                GCHandle handle = GCHandle.Alloc(subData, GCHandleType.Pinned);
                try
                {
                    result[i] = (T)Marshal.PtrToStructure(handle.AddrOfPinnedObject(), typeof(T));
                }
                finally
                {
                    handle.Free();
                }
            }

            return result;
        }


        

    }
}
